@@ -36,7 +36,7 @@
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.4'
+__version__ = '0.5'
 __website__ = 'https://github.com/stefankoegl/python-json-pointer'
 __license__ = 'Modified BSD License'
 
@@ -112,14 +112,14 @@ def set_pointer(doc, pointer, value):
     >>> pointer = JsonPointer('/bar')
     >>> pointer.set(obj, 'one', '0')
     >>> pointer.set(obj, 'two', '1')
-    >>> obj
-    {'foo': 2, 'bar': ['one', 'two']}
+    >>> obj == {'foo': 2, 'bar': ['one', 'two']}
+    True
 
     >>> obj = {"foo": 2, "bar": []}
     >>> pointer = JsonPointer('/bar')
     >>> pointer.set(obj, 5, '0/x')
-    >>> obj
-    {'foo': 2, 'bar': [{'x': 5}]}
+    >>> obj == {'foo': 2, 'bar': [{'x': 5}]}
+    True
 
     >>> obj = {'foo': 2, 'bar': [{'x': 5}]}
     >>> pointer = JsonPointer('/bar/0')
@@ -146,6 +146,14 @@ class JsonPointer(object):
         self.parts = parts
 
 
+    def to_last(self, doc, default=_nothing):
+        """ Resolves ptr until the last step, returns (sub-doc, last-step) """
+
+        for part in self.parts[:-1]:
+            doc = self.walk(doc, part)
+
+        return doc, self.get_part(doc, self.parts[-1])
+
 
     def resolve(self, doc, default=_nothing):
         """Resolves the pointer against doc and returns the referenced object"""
@@ -208,9 +216,31 @@ class JsonPointer(object):
         return doc[part]
 
 
+    def get_part(self, doc, part):
+        """ Returns the next step in the correct type """
+
+        if isinstance(doc, dict):
+            return part
+
+        elif isinstance(doc, list):
+
+            if part == '-':
+                return part
+
+            try:
+                return int(part)
+            except ValueError:
+                raise JsonPointerException("'%s' is not a valid list index" % (part, ))
+
+        else:
+            raise JsonPointerException("Unknown document type '%s'" % (doc.__class__,))
+
+
     def walk(self, doc, part):
         """ Walks one step in doc and returns the referenced part """
 
+        part = self.get_part(doc, part)
+
         if isinstance(doc, dict):
             try:
                 return doc[part]
@@ -223,11 +253,6 @@ class JsonPointer(object):
             if part == '-':
                 return EndOfList(doc)
 
-            try:
-                part = int(part)
-            except ValueError:
-                raise JsonPointerException("'%s' is not a valid list index" % (part, ))
-
             try:
                 return doc[part]
 
@@ -240,6 +265,12 @@ class JsonPointer(object):
 
 
 
+    def contains(self, ptr):
+        """" Returns True if self contains the given ptr """
+        return len(self.parts) > len(ptr.parts) and \
+             self.parts[:len(ptr.parts)] == ptr.parts
+
+
 def pairwise(iterable):
     "s -> (s0,s1), (s1,s2), (s2, s3), ..."
     a, b = tee(iterable)
